use crate::data::{Config, MyState, SteepingLog, Tea};
use axum::{
    extract::{Path, State},
    http::StatusCode,
    response::IntoResponse,
    Json,
};

pub async fn retrieve_config(
    State(state): State<MyState>,
) -> Result<impl IntoResponse, impl IntoResponse> {
    match sqlx::query_as::<_, Config>("SELECT * FROM config")
        .fetch_one(&state.pool)
        .await
    {
        Ok(config) => Ok((StatusCode::OK, Json(config))),
        Err(e) => Err((StatusCode::BAD_REQUEST, e.to_string())),
    }
}

pub async fn retrieve_types_of_tea(
    State(state): State<MyState>,
) -> Result<impl IntoResponse, impl IntoResponse> {
    match sqlx::query_as::<_, Tea>("SELECT * FROM tea LIMIT 400")
        .fetch_all(&state.pool)
        .await
    {
        Ok(all_tea) => Ok((StatusCode::OK, Json(all_tea))),
        Err(e) => Err((StatusCode::BAD_REQUEST, e.to_string())),
    }
}

pub async fn retrieve_tea_by_id(
    State(state): State<MyState>,
    Path(id): Path<i32>,
) -> Result<impl IntoResponse, impl IntoResponse> {
    match sqlx::query_as::<_, Tea>("SELECT * FROM tea WHERE id = $1 LIMIT 1")
        .bind(id)
        .fetch_optional(&state.pool)
        .await
    {
        Ok(tea) => Ok((StatusCode::OK, Json(tea))),
        Err(e) => Err((StatusCode::BAD_REQUEST, e.to_string())),
    }
}

pub async fn retrieve_brewing_events(
    State(state): State<MyState>,
) -> Result<impl IntoResponse, impl IntoResponse> {
    match sqlx::query_as::<_, SteepingLog>(
        "
        SELECT *
        FROM steepinglog
        ORDER BY tea_id
        ",
    )
    .fetch_all(&state.pool)
    .await
    {
        Ok(steeping_log) => Ok((StatusCode::OK, Json(steeping_log))),
        Err(e) => Err((StatusCode::BAD_REQUEST, e.to_string())),
    }
}

// used by teavail-device
pub async fn retrieve_tea_by_rfid(
    Path(rfid): Path<String>,
    State(state): State<MyState>,
) -> Result<impl IntoResponse, impl IntoResponse> {
    match sqlx::query_as::<_, Tea>("SELECT * FROM tea WHERE rfid_code = $1 LIMIT 1")
        .bind(rfid)
        .fetch_optional(&state.pool)
        .await
    {
        Ok(tea) => Ok((StatusCode::OK, Json(tea))),
        Err(e) => Err((StatusCode::BAD_REQUEST, e.to_string())),
    }
}

// ---------------- currently not in use ---------------

pub async fn delete_tea_by_id(
    Path(id): Path<i32>,
    State(state): State<MyState>,
) -> Result<impl IntoResponse, impl IntoResponse> {
    match sqlx::query_as::<_, Tea>(
        "
        DELETE FROM tea WHERE id = $1
        RETURNING id, tea_name, rfid_code, tea_notes, water_temp, steeping_seconds, registration_timestamp
        ",
    )
    .bind(id)
    .fetch_one(&state.pool)
    .await
    {
        Ok(tea) => Ok((StatusCode::OK, Json(tea))),
        Err(e) => Err((StatusCode::BAD_REQUEST, e.to_string())),
    }
}
